% Newton's method for 2-point Gauss quadrature rule
% Exact solution is x1 = -1/sqrt(3), x2 = 1/sqrt(3), w1 = w2 = 1
clear all; clc;

F = @(x)([x(3) + x(4) - 2
          x(3)*x(1) + x(4)*x(2)
          x(3)*x(1)^2 + x(4)*x(2)^2 - 2/3
          x(3)*x(1)^3 + x(4)*x(2)^3]);
jacobian_function = @(x)([0             0             1      1
                          x(3)          x(4)          x(1)   x(2)
                          2*x(3)*x(1)   2*x(4)*x(2)   x(1)^2 x(2)^2
                          3*x(3)*x(1)^2 3*x(4)*x(2)^2 x(1)^3 x(2)^3]);

% set the inital guess
x_0 = [-1,1,1,1]';
x_k = x_0;

step_size = inf;
TOL = 1.e-8;

% perform Newton iterations until tolerance is satisfied
while (step_size > TOL)
    J_F = jacobian_function(x_k);
    
    Delta_x_k = -J_F\F(x_k);
    
    x_k = x_k + Delta_x_k;
    
    step_size = norm(Delta_x_k)/norm(x_k)
end

fprintf('Newton method gives x=(%d,%d), w=(%d,%d)\n', ...
    x_k(1),x_k(2),x_k(3),x_k(4))

fsolve_root = fsolve(F,x_0);
fprintf('fsolve gives x=(%d,%d), w=(%d,%d)\n', ...
    fsolve_root(1),fsolve_root(2),fsolve_root(3),fsolve_root(4))